@@ -47,17 +47,6 @@ class BaseRepository(metaclass=ABCMeta):
         Monkey patches pip.Wheel to allow wheels from all platforms and Python versions.
         """
 
-    @abstractmethod
-    def copy_ireq_dependencies(
-        self, source: InstallRequirement, dest: InstallRequirement
-    ) -> None:
-        """
-        Notifies the repository that `dest` is a copy of `source`, and so it
-        has the same dependencies. Otherwise, once we prepare an ireq to assign
-        it its name, we would lose track of those dependencies on combining
-        that ireq with others.
-        """
-
     @property
     @abstractmethod
     def options(self) -> optparse.Values:
